#!/usr/bin/perl
#
#  Voicemail transcription
#
#  This script sends all voicemail messages off for transcription, if
#  customer has the feature enabled.
#

use strict;
use Sys::Hostname;


########## DEBUG ############################################
my $DEBUG = 0;
my $debug_cgi_host = 'cp.test17.trixbox.com';
#############################################################


# parameters Asterisk sends us (see externnotify in voicemail.conf)
my ($context, $mailbox, $message_count) = @ARGV;

# folder containing all VM for given mailbox
my $path = '/var/spool/asterisk/voicemail/'.$context.'/'.$mailbox.'/INBOX';

# This is what Asterisk would normally use, for the mailcmd (voicemail.conf)
# Must be a command that accepts an email via stdin (pipe)
# This is only used for fallback, in case the CP can't be contacted.
my $asterisk_default_mailcmd = '/usr/sbin/sendmail';

# trixbox or PBXtra/Connect?
open(VPN, "</etc/sysconfig/vpn");
my $stype = 'pbxtra';
while (<VPN>)
{
	if (/^GW.*trixbox/)
	{
		$stype = 'trixbox';
		last;
	}
}
close(VPN);

my ($server_id) = hostname;
$server_id =~ s/(?:trixbox|pbxtra)//;

# This is for UNBOUND/Connect
if ($context =~ /-(\d+)$/)
{
	$server_id = $1;
}


my $cgi_host = ($DEBUG ? $debug_cgi_host : 'cp.' . ($stype eq 'trixbox' ? 'trixbox' : 'fonality') . '.com');
my $process_cgi = ($DEBUG ? 'http' : 'https') . "://$cgi_host/vm_notify.cgi";


if (-e $path)
{
	# sort files by timestamp -- only grab metadata (.txt)
	# warning: 
	#	We're making a bit of an assumption here and hoping there is no race condition.
	#   	Asterisk passes us no identification of the actual message itself, so all we can do
	#	is grab the newest message found and hope it's what we need.
	my @messages = `/bin/ls -1t $path/*.txt`;
	if (@messages)
	{
		my $last_message = $messages[0];
		$last_message = ($last_message =~ /\/(msg\d+\.txt)\n?$/) ? $1 : '';

		my $url = $process_cgi . "?sid=$server_id&mb=$mailbox&context=$context&metadata=$last_message";

		debug("last_message = '$last_message'");
		debug("url = '$url'");
		
		my $transcribed_vm = do_wget($url);

		debug("transcribed_vm = '$transcribed_vm'");

		# Used for email files that Asterisk generates (via mailcmd in voicemail.conf)
		my $email_file = $last_message;
		$email_file =~ s/\.txt$//;

		if (!$transcribed_vm)
		{
			# We probably couldn't connect to the CP, so just send a default
			# voicemail notification. 
			my $i = 0;
			while (!<$path/$email_file*.email>)
			{
				sleep 5;
				$i++;

				# We can't send an email if Asterisk didn't generate one.
				# This is not an error; user probably just doesn't have
				# VM notifications turned on.
				exit if $i > 5;
			}

			# Send emails
			foreach my $file (<$path/$email_file*.email>)
			{
				`cat $file | $asterisk_default_mailcmd`;
			}
		}

		# Delete all mail notifications.  These have already been
		# handled by the CP or by us
		foreach my $file (<$path/$email_file*.email>)
		{
			unlink $file;
		}
	}
}



# Use wget rather than Perl here.  Premise boxes don't have the correct modules
# for dealing with SSL in Perl, and we don't want to risk any redirection to SSL causing issues.
sub do_wget
{
	my ($url) = @_;
	
	# file for temporary wget output
	my $tmpfile = `/bin/mktemp`;
	chomp $tmpfile;
	
	my $timeout_secs = 60 * 2;
	my $wget = '/usr/bin/wget';
	my $args = "-t 1 -T $timeout_secs -O $tmpfile '$url'";
	my $cmd = "$wget $args >/dev/null 2>&1";
	
	unless (system($cmd) == 0)
	{
		$@ = "Could not fetch URL: $url";
		return undef;
	}
	
	unless (open INP, "<$tmpfile")
	{
		$@ = "Could not open $tmpfile for reading.";
		return undef;
	}
	
	my @lines = <INP>;
	close INP;
	unlink $tmpfile;
	
	return join('', @lines);
}


sub debug
{
	my ($msg) = @_;

	if ($DEBUG)
	{
		my $ts = `date +'%x %T'`;
		chomp $ts;
		open OUT, ">>/tmp/vm_notify_debug.txt";
		print OUT "[$ts] $msg\n";
		close OUT;
	}
}
